/* Copyright (C) 2005 Juergen "George" Sawinski
 * All rights reserved.
 * Licensed under the LGPL v2 or later.
 * See accompanying COPYING.LIB file.
 */
#ifndef _TAR_H_
#define _TAR_H_

#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include <libtar.h>

#define LibName      "tar"
#define LibInitFunc  luaopen_ltar
#define LibType      "Tar"
#define LibVersion   "0.0"
#define LibCopyright "Copyright (C) 2005 Juergen \"George\" Sawinski"
#define LibDesc      "Tar module"

/*-----------------------------------------------------------------------------*\
| LUA userdata                                                                  |
\*-----------------------------------------------------------------------------*/
/* TAR userdata */
typedef struct __tar_ud tar_ud;
struct __tar_ud {
  /* LUA state and reference */
  lua_State  *L;
  int         ref;
  
  /* tar file handle */
  TAR        *file;
  
  /* flags */
  int read_only;   /* read-only */
  int stream_type; /* see enum __luatar_stream_type */

#define LUATAR_STATUS_ERROR    -1 /* an error occured */
#define LUATAR_STATUS_INIT      0 /* initial status */
#define LUATAR_STATUS_HEADER    1 /* header was read/written */
#define LUATAR_STATUS_FILE      2 /* file was read/written */
  int status; /* tar status */
};

#ifndef LUATAR_GET_LIMIT
#define LUATAR_GET_LIMIT     64*1024   /* limit for extracting file content as text into a variable */
#endif

/*-----------------------------------------------------------------------------*\
| Compression                                                                   |
\*-----------------------------------------------------------------------------*/
LUA_API void luatar_cstream_add(int fd, void *F);

extern tartype_t c_gzip;
extern tartype_t c_bzip2;

/*-----------------------------------------------------------------------------*\
| Auxilliary                                                                    |
\*-----------------------------------------------------------------------------*/
enum __luatar_stream_type {
  LUATAR_TYPE_PLAIN,
  LUATAR_TYPE_ZIP,
  LUATAR_TYPE_GZIP,
  LUATAR_TYPE_BZIP2,
};

LUA_API int luatar_filemagic(const char *fname);
LUA_API int luatar_filetype(const char *typestring);

LUA_API int aux_luatar_open(tar_ud *ud, const char *fname, int type, int flags);
LUA_API int aux_luatar_new(tar_ud *ud, const char *fname, int type, int compression, int flags, int mode);
LUA_API int aux_luatar_close(tar_ud *ud);

/*-----------------------------------------------------------------------------*\
| Debug                                                                         |
\*-----------------------------------------------------------------------------*/
#ifdef DEBUG
#include <stdio.h>
#define dbg(fmt, args...) fprintf(stderr, __FILE__ "[%d]: " fmt, __LINE__, ## args)
#include <ctype.h>
extern inline void dbgXDump(const char *buf, size_t len) 
{
  int n,k;
  for(n=0; n<len; n+=16) {
    fprintf(stderr, "%06X | ", n);
    for(k=0; k<16; k++) {
      if (k==8) fprintf(stderr,"| ");
      if (n+k<len)
	fprintf(stderr, "%02X ", buf[n+k]);
      else
	fprintf(stderr, "   ");
    }
    fprintf(stderr,"|");
    for(k=0; k<16; k++) {
      if (k==8) fprintf(stderr,"|");
      if (n+k<len) {
	if (isprint(buf[n+k])) fprintf(stderr, "%c", buf[n+k]);
	else fprintf(stderr, ".");
      }
      else
	fprintf(stderr, " ");
    }
    fprintf(stderr, "\n");
  }
}
#else
#define dbg(fmt, args...) do {} while(0)
#define dbgXDump(buf, len)
#endif

#endif
